"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const { NODE_ENV } = process.env;
const isDev = !NODE_ENV || NODE_ENV === 'development';
const isTest = NODE_ENV === 'test';
const devMemo = (Com, compareProps = [], options = {}) => {
    const opts = {
        outline: '1px solid #f2e',
        duration: 500,
        highlight: isDev || isTest,
        console: false,
        ...options,
    };
    const InjectedHighlight = props => {
        const [style, setStyle] = react_1.default.useState({});
        react_1.default.useEffect(() => {
            setStyle({
                outline: opts.outline,
            });
            const t = setTimeout(() => {
                setStyle({});
            }, opts.duration);
            return () => clearTimeout(t);
        }, compareProps.length
            ? compareProps.map(name => props[name])
            : Object.values(props));
        return react_1.default.createElement(Com, Object.assign({}, props, { style: { ...props.style, ...style } }));
    };
    return react_1.default.memo(opts.highlight ? InjectedHighlight : Com, (prevProps, nextProps) => {
        const noUpdate = compareProps.length
            ? compareProps.every(propsName => prevProps[propsName] === nextProps[propsName])
            : Object.keys(prevProps).every(propsName => prevProps[propsName] === nextProps[propsName]);
        if (opts.console && !noUpdate) {
            const { displayName = (Com.toString()
                .split('\n')[0]
                .match(/function\s([^(]+)/) || [])[1], } = Com;
            console.warn(`%c[devMemo]%c update ${displayName}`, 'color: yellow', 'color: inherit');
            console.table({
                prev: prevProps,
                next: nextProps,
            });
            let diffProps = [];
            if (compareProps.length) {
                diffProps = compareProps.filter(propName => prevProps[propName] !== nextProps[propName]);
            }
            else {
                diffProps = Object.keys(prevProps).filter(propName => prevProps[propName] !== nextProps[propName]);
            }
            console.log('Diff props:', diffProps);
            console.table(diffProps.reduce((o, propName) => Object.assign(o, {
                [propName]: {
                    prev: prevProps[propName],
                    next: nextProps[propName],
                },
            }), {}));
        }
        return noUpdate;
    });
};
exports.default = devMemo;
